# move a singing bird with the arrow keys
# press escape to exit

import time

# import the pygame module
import pygame


class Bird:

    x = .5 # left = 0, right = 1, center = .5
    speed = 0 # distance by second

    def __init__(self):
        self._previous_update = time.time()
        self._previous_sound = time.time()
        # always use the same channel, so only one sound
        # will be active at a time
        self._channel = pygame.mixer.Channel(0)
        # load a sound file into memory
        self._sound = pygame.mixer.Sound("bird.ogg")

    def update(self):
        # update the coordinates
        self.x += (time.time() - self._previous_update) * self.speed
        if self.x > 1:
            self.x = 1
        elif self.x < 0:
            self.x = 0
        self._previous_update = time.time()

    def render(self):
        # make a noise every .1 seconds (if moving) or .5 seconds (if still)
        # (the result isn't very precise, it seems; strange)
        interval = .1 if self.speed else .5
        if time.time() >= self._previous_sound + interval:
            # in theory the previous sound should stop at once
            # but the sound plays completely ; strange
            self._channel.stop()
            self._channel.play(self._sound)
            self._channel.set_volume(1 - self.x, self.x)
            self._previous_sound = time.time()


# start pygame
pygame.init()

# start the display (required by the event loop)
pygame.display.set_mode((320, 200))

# create the bird object
bird = Bird()

# loop forever (until a break occurs)
while True:
    # get every event from the event queue
    for event in pygame.event.get():
        # if the event is about a keyboard button that have been pressed...
        if event.type == pygame.KEYDOWN:
            if event.key == pygame.K_LEFT:
                bird.speed = -.2
            elif event.key == pygame.K_RIGHT:
                bird.speed = .2
            # and if the button is the "q" letter or the "escape" key...
            elif event.unicode == "q" or event.key == pygame.K_ESCAPE:
                # ... then exit from the while loop
                # (break would not work here because of the for loop)
                raise SystemExit
        # if the event is about a keyboard button that have been depressed...
        elif event.type == pygame.KEYUP:
            if event.key in (pygame.K_LEFT, pygame.K_RIGHT):
                bird.speed = 0
    # update the simulation of the bird
    bird.update()
    # render the bird (in an audio way)
    bird.render()
    # do nothing for some time
    # (without this the CPU usage would be 100%)
    time.sleep(.001)
